/*
 * expand SD card sector size from 128bytes to 512 bytes
 * Ver 0.01 2018/03/12 by skyriver
 * Ver 0.02 2018/03/17 by skyriver
 *  inprement reversely transration
 */

#include <stdio.h>
#include <string.h>

#define	SEC_SIZE	512	// SD sector size
#define	ORG_SIZE	128	// original sector size


char *InputFname;		// input file name
int RevFlg;
FILE *RdFp, *WrFp;
char FnameBuf[ 128 ];

char *Help =
	"SD image splitter : expand original sector size(128bytes) to 512bytes\n"
	"usage : [-r] InputFileName\n"
	"  -r : translate reversely\n"
	"  ver 0.02 2018/03/17 by skyriver\n"
;

char *FnameForm[] = {
	 "%s_exp.img"	// expand
	,"%s_rev.img"	// reverse
};


char *FindDot( char *str )
{
	char *retval;

	retval = NULL;
	for( ; *str; str++ ) {
		if( *str == '.' ) {
			retval = str;
		}
	}
	if( retval == NULL ) {
		retval = str;
	}
	return( retval );
}


int Expand( void )
{
	int retval, c;
	char *pnt;

	retval = 0;
	pnt = FindDot( InputFname );
	*pnt = '\0';
	sprintf( FnameBuf, FnameForm[ RevFlg ], InputFname );
	if( ( WrFp = fopen( FnameBuf, "wb" ) ) == NULL ) {
		retval = 10;
		fprintf( stderr, "cann't open [%s]\n", FnameBuf );
	} else {
		fprintf( stderr, "... creating [%s]", FnameBuf );
		do  {
			int i;
			for( i = 0; i < ORG_SIZE; i++ ) {
				if( ( c = getc( RdFp ) ) == EOF ) {
					break;
				} else {
					putc( c, WrFp );
				}
			}
			if( c != EOF ) {
				for( ; i < SEC_SIZE; i++ ) {
					if( RevFlg == 0 ) {		// expand
						putc( 0xe5, WrFp );
					} else {
						if( getc( RdFp ) == EOF ) {
							retval = 11;
							break;
						}
					}
				}
			}
		} while( c != EOF );
		fclose( WrFp );
		fprintf( stderr, "  complete!\n" );
	}
	return( retval );
}


void ErrExit( char *msg )
{
	fprintf( stderr, "err at [%s]\n", msg );
}


int main( int argc, char **argv )
{
	char *pnt, *myname;
	int i, retval;
	
	retval = 0;
	myname = *argv++;
	for ( i = 1; i < argc; i++ ) {
		pnt = *argv++;
		switch( *pnt ) {
		case '-':
			switch( *++pnt ) {
			case 'r':
				RevFlg = 1;
				break;
			default:
				ErrExit( pnt );
				retval = 1;
				break;
			}
			break;
		default:
			if( InputFname ) {
				ErrExit( pnt );
				retval = 2;
			} else {
				InputFname = pnt;
			}
			break;
		}
	}
	if( retval == 0 ) {
		if( InputFname == NULL ) {
			fprintf( stderr, Help, myname );
			retval = 3;
		} else {
			if( ( RdFp = fopen( InputFname, "rb" ) ) == NULL ) {
				fprintf( stderr, "cann't open [%s]\n", InputFname );
				retval = 4;
			} else {
				retval = Expand();	
			}
			fclose( RdFp );
		}
	}
	return( retval );
}
